# frozen_string_literal: true

require "git/worklog/version"
require "thor"

module Utilities

  def default_git_username
    `git config user.name`.strip
  end

  def default_git_email
    `git config user.email`.strip
  end

  def default_git_user
    "#{default_git_username} <#{default_git_email}>"
  end

  def repos
    `find . -type d -exec test -e '{}/.git' ';' -print -prune`.split("\n")
  end

  def pwd
    `pwd`
  end

  def root_repository_name
    pwd.split('/').last
  end

  def user(name)
    if !name.nil?
      return name
    else
      return default_git_user
    end
  end
end

module GitCLIParser

end

module Worklog
  class CLI < Thor
    package_name "git-worklog"
    include Thor::Actions
    include Utilities

    def self.exit_on_failure?
      true
    end

    desc "show", "Shows worklog from current repository/directory (Default: Last 7 days)"
    method_option :days, :aliases => "-d", :desc => "Specify number of days"
    method_option :weeks, :aliases => "-w", :desc => "Specify number of weeks"
    method_option :user, :aliases => "-u", :desc => "Specify Author name"
    method_option :path, :aliases => "-p", :desc => "Specify path from where to check for the repos"
    method_option :format, :aliases => "-f", :desc => "Oneline, short, medium, full, fuller and raw"
    method_option :reverse, :aliases => "-r", :desc => "Show output of logs in reverse"
    method_option :type, :aliases => "-t", :desc => "Merge Commits, Branch Commits"
    method_option :repositories, :aliases => "-repos", :desc => "Show commits only from these repositories"

    def show
      commits = {}
      git_command = "git --no-pager log --exclude=refs/stash --pretty=oneline --date=short --since=#{options[:days] || 7}.days --author=\"#{user(options[:user])}\" --all"
      repos.each do |repo|
        repo_name = repo.split("/").last
        output = `cd #{repo} && #{git_command}`
        commits[repo_name] = output
        system("cd #{pwd}")
      end
      commits = commits.filter { |key, value| value != ""}
      commits.map do |key, value|
        if key == "."
          puts "\n\n---------------- PROJECT: #{root_repository_name} -----------------\n\n"
        else
          puts "\n\n---------------- PROJECT: #{key} -----------------\n\n"
        end
        puts value
      end
    end
  end
end

Worklog::CLI.start(ARGV)
